@@ -1,4 +1,20 @@
-/*
+/*  Copyright 1999 Peter Schlaile.
+ *  Copyright 1999-2005,2007-2009 Alain Knaff.
+ *  This file is part of mtools.
+ *
+ *  Mtools is free software: you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation, either version 3 of the License, or
+ *  (at your option) any later version.
+ *
+ *  Mtools is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with Mtools.  If not, see <http://www.gnu.org/licenses/>.
+ *
  * the floppyd daemon running on the local X-Server
  *
  * written by:
@@ -580,7 +596,11 @@ static uid_t getuserid(char *user)
 		}
 	else
 		{
+#ifdef HAVE_GETUSERID
+			id = getuserid("nobody");
+#else
 			uid = 65535;
+#endif
 		}
 
 #if DEBUG
@@ -606,7 +626,11 @@ static uid_t getgroupid(uid_t uid)
 		}
 	else
 		{
+#ifdef HAVE_GETGROUPID
+			id = getgroupid(uid);
+#else
 			gid = 65535;
+#endif
 		}
 
 #if DEBUG
@@ -741,7 +765,7 @@ static void server_main_loop(int sock, char **device_name, int n_dev)
 /*
  * Print some basic help information.
  */
-static void usage(char *prog, const char *opt)
+static void usage(char *prog, const char *opt, int ret)
 {
 	if (opt)
 		{
@@ -754,7 +778,7 @@ static void usage(char *prog, const char *opt)
 	fprintf(stderr, "    -r user     Run as the specified user in server mode.\n");
 	fprintf(stderr, "    -b ipaddr   Bind to the specified ipaddr in server mode.\n");
 	fprintf(stderr, "    -l          Do not attempt to connect to localhost:0 to validate connection\n");
-	exit(1);
+	exit(ret);
 }
 
 
@@ -780,14 +804,16 @@ int main (int argc, char** argv)
 
 	char *server_hostname=NULL;
 	char **device_name = NULL; 
-	char *floppy0 = "/dev/fd0";
+	const char *floppy0 = "/dev/fd0";
 	int n_dev;
 
 
 	/*
 	 * Parse the command line arguments.
 	 */
-	while ((arg = getopt(argc, argv, "ds:r:b:x:")) != EOF)
+	if(argc > 1 && !strcmp(argv[0], "--help"))
+		usage(argv[0], NULL, 0);
+	while ((arg = getopt(argc, argv, "ds:r:b:x:h")) != EOF)
 		{
 			switch (arg)
 				{
@@ -815,8 +841,11 @@ int main (int argc, char** argv)
 						dispName = strdup(optarg);
 						break;
 
+					case 'h':
+						usage(argv[0], NULL, 0);
+						break;
 					case '?':
-						usage(argv[0], NULL);
+						usage(argv[0], NULL, 1);
 						break;
 				}
 		}
@@ -825,7 +854,7 @@ int main (int argc, char** argv)
 		device_name = argv + optind;
 		n_dev = argc - optind;
 	} else {
-		device_name = &floppy0;
+		device_name = (char **)&floppy0;
 		n_dev = 1;
 	}
 
@@ -862,10 +891,10 @@ int main (int argc, char** argv)
 	 * Test to make sure required args were provided and are valid.
 	 */
 	if (run_as_server && (bind_ip == INADDR_NONE)) {
-		usage(argv[0], "The server ipaddr is invalid.");
+		usage(argv[0], "The server ipaddr is invalid.", 1);
 	}
 	if (run_as_server && (bind_port == 0))	{
-		usage(argv[0], "No server port was specified (or it was invalid).");
+		usage(argv[0], "No server port was specified (or it was invalid).", 1);
 	}
 
 
@@ -917,11 +946,13 @@ int main (int argc, char** argv)
 					 * Start a new session and group.
 					 */
 					setsid();
+#ifdef HAVE_SETPGRP
 #ifdef SETPGRP_VOID
 					setpgrp();
 #else
 					setpgrp(0,0);
 #endif
+#endif
 #if DEBUG
 					close(2);
 					open("/dev/null", O_WRONLY);
