@@ -1,19 +1,29 @@
+/*  Copyright 1999-2003,2006,2008,2009 Alain Knaff.
+ *  This file is part of mtools.
+ *
+ *  Mtools is free software: you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation, either version 3 of the License, or
+ *  (at your option) any later version.
+ *
+ *  Mtools is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with Mtools.  If not, see <http://www.gnu.org/licenses/>.
+ */
+
 #include "sysincludes.h"
 #include "stream.h"
 #include "fsP.h"
 #include "llong.h"
 #include "mtools.h"
 
-/* Warnings about integer overflow in expression can be ignored.  These are
- * due to the way that maximal values for those integers are computed: 
- * intentional overflow from smallest negative number (1000...) to highest 
- * positive number (0111...) by substraction of 1 */
-#ifdef __GNUC__
-#warning "The following warnings about integer overflow in expression can be safely ignored"
-#endif
-
 #if 1
 const mt_off_t max_off_t_31 = MAX_OFF_T_B(31); /* Floppyd */
+const mt_off_t max_off_t_32 = MAX_OFF_T_B(32); /* Directory */
 const mt_off_t max_off_t_41 = MAX_OFF_T_B(41); /* SCSI */
 const mt_off_t max_off_t_seek = MAX_OFF_T_B(SEEK_BITS); /* SCSI */
 #else
@@ -22,9 +32,13 @@ const mt_off_t max_off_t_41 = MAX_OFF_T_B(10); /* SCSI */
 const mt_off_t max_off_t_seek = MAX_OFF_T_B(10); /* SCSI */
 #endif
 
+int fileTooBig(mt_off_t off) {
+	return (off & ~max_off_t_32) != 0;
+}
+
 off_t truncBytes32(mt_off_t off)
 {
-	if (off & ~max_off_t_31) {
+	if (fileTooBig(off)) {
 		fprintf(stderr, "Internal error, offset too big\n");
 		exit(1);
 	}
@@ -70,9 +84,9 @@ int mt_lseek(int fd, mt_off_t where, int whence)
 #endif
 }
 
-int log_2(int size)
+unsigned int log_2(int size)
 {
-	int i;
+	unsigned int i;
 
 	for(i=0; i<24; i++) {
 		if(1 << i == size)
