@@ -1,4 +1,20 @@
-/*
+/*  Copyright 1986-1992 Emmet P. Gray.
+ *  Copyright 1996-1998,2000-2002,2007,2009 Alain Knaff.
+ *  This file is part of mtools.
+ *
+ *  Mtools is free software: you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation, either version 3 of the License, or
+ *  (at your option) any later version.
+ *
+ *  Mtools is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with Mtools.  If not, see <http://www.gnu.org/licenses/>.
+ *
  * mattrib.c
  * Change MSDOS file attribute flags
  */
@@ -116,15 +132,15 @@ static int recursive_attrib(direntry_t *entry, MainParam_t *mp)
 }
 
 
-static void usage(void) NORETURN;
-static void usage(void)
+static void usage(int ret) NORETURN;
+static void usage(int ret)
 {
 	fprintf(stderr, "Mtools version %s, dated %s\n", 
 		mversion, mdate);
 	fprintf(stderr, 
 		"Usage: %s [-p] [-a|+a] [-h|+h] [-r|+r] [-s|+s] msdosfile [msdosfiles...]\n",
 		progname);
-	exit(1);
+	exit(ret);
 }
 
 static int letterToCode(int letter)
@@ -139,7 +155,7 @@ static int letterToCode(int letter)
 		case 'S':
 			return ATTR_SYSTEM;
 		default:
-			usage();
+			usage(1);
 	}
 }
 
@@ -161,7 +177,9 @@ void mattrib(int argc, char **argv, int type)
 	concise = 0;
 	replay = 0;
 	
-	while ((c = getopt(argc, argv, "i:/ahrsAHRSXp")) != EOF) {
+	if(helpFlag(argc, argv))
+		usage(0);
+	while ((c = getopt(argc, argv, "i:/ahrsAHRSXph")) != EOF) {
 		switch (c) {
 			default:
 				arg.remove &= ~letterToCode(c);
@@ -178,8 +196,10 @@ void mattrib(int argc, char **argv, int type)
 			case 'X':
 				concise = 1;
 				break;
+			case 'h':
+				usage(0);
 			case '?':
-				usage();
+				usage(1);
 		}
 	}
 
@@ -201,7 +221,7 @@ void mattrib(int argc, char **argv, int type)
 		view = 1;
 
 	if (optind >= argc)
-		usage();
+		usage(1);
 
 	init_mp(&arg.mp);
 	if(view){
