@@ -1,4 +1,20 @@
-/*
+/*  Copyright 1986-1992 Emmet P. Gray.
+ *  Copyright 1996-1998,2000-2002,2005,2007-2009 Alain Knaff.
+ *  This file is part of mtools.
+ *
+ *  Mtools is free software: you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation, either version 3 of the License, or
+ *  (at your option) any later version.
+ *
+ *  Mtools is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with Mtools.  If not, see <http://www.gnu.org/licenses/>.
+ *
  * mlabel.c
  * Make an MSDOS volume label
  */
@@ -9,41 +25,48 @@
 #include "vfat.h"
 #include "mtools.h"
 #include "nameclash.h"
+#include "file_name.h"
 
-char *label_name(char *filename, int verbose, 
-		 int *mangled, char *ans)
+void label_name(doscp_t *cp, const char *filename, int verbose,
+		int *mangled, dos_name_t *ans)
 {
 	int len;
 	int i;
 	int have_lower, have_upper;
+	wchar_t wbuffer[12];
 
-	strcpy(ans,"           ");
-	len = strlen(filename);
+	strcpy(ans->base,"           ");
+	len = native_to_wchar(filename, wbuffer, 11, 0, 0);
 	if(len > 11){
 		*mangled = 1;
 		len = 11;
 	} else
 		*mangled = 0;
-	strncpy(ans, filename, len);
-	have_lower = have_upper = 0;
-	for(i=0; i<11; i++){
-		if(islower((unsigned char)ans[i]))
-			have_lower = 1;
-		if(isupper(ans[i]))
-			have_upper = 1;
-		ans[i] = toupper((unsigned char)ans[i]);
 
-		if(strchr("^+=/[]:,?*\\<>|\".", ans[i])){
+	have_lower = have_upper = 0;
+	for(i=0; i<len; i++){
+		if(islower(wbuffer[i]))
+			have_lower = 1;
+		if(isupper(wbuffer[i]))
+			have_upper = 1;
+		wbuffer[i] = towupper(wbuffer[i]);
+		if(
+#ifdef HAVE_WCHAR_H
+		   wcschr(L"^+=/[]:,?*\\<>|\".", wbuffer[i])
+#else
+		   strchr("^+=/[]:,?*\\<>|\".", wbuffer[i])
+#endif
+		   ){
 			*mangled = 1;
-			ans[i] = '~';
+			wbuffer[i] = '~';
 		}
 	}
 	if (have_lower && have_upper)
 		*mangled = 1;
-	return ans;
+	wchar_to_dos(cp, wbuffer, ans->base, len, mangled);
 }
 
-int labelit(char *dosname,
+int labelit(struct dos_name_t *dosname,
 	    char *longname,
 	    void *arg0,
 	    direntry_t *entry)
@@ -56,24 +79,25 @@ int labelit(char *dosname,
 	return 0;
 }
 
-static void usage(void)
+static void usage(int ret) NORETURN;
+static void usage(int ret)
 {
 	fprintf(stderr, "Mtools version %s, dated %s\n",
 		mversion, mdate);
 	fprintf(stderr, "Usage: %s [-vscVn] [-N serial] drive:\n", progname);
-	exit(1);
+	exit(ret);
 }
 
 
 void mlabel(int argc, char **argv, int type)
 {
-    
+
 	char *newLabel;
 	int verbose, clear, interactive, show;
 	direntry_t entry;
 	int result=0;
 	char longname[VBUFSIZE];
-	char shortname[13];
+	char shortname[45];
 	ClashHandling_t ch;
 	struct MainParam_t mp;
 	Stream_t *RootDir;
@@ -99,7 +123,9 @@ void mlabel(int argc, char **argv, int type)
 	clear = 0;
 	show = 0;
 
-	while ((c = getopt(argc, argv, "i:vcsnN:")) != EOF) {
+	if(helpFlag(argc, argv))
+		usage(0);
+	while ((c = getopt(argc, argv, "i:vcsnN:h")) != EOF) {
 		switch (c) {
 			case 'i':
 				set_cmd_line_image(optarg, 0);
@@ -128,13 +154,15 @@ void mlabel(int argc, char **argv, int type)
 					exit(1);
 				}
 				break;
+			case 'h':
+				usage(0);
 			default:
-				usage();
+				usage(1);
 			}
 	}
 
-	if (argc - optind != 1 || !argv[optind][0] || argv[optind][1] != ':') 
-		usage();
+	if (argc - optind != 1 || !argv[optind][0] || argv[optind][1] != ':')
+		usage(1);
 
 	init_mp(&mp);
 	newLabel = argv[optind]+2;
@@ -144,7 +172,7 @@ void mlabel(int argc, char **argv, int type)
 		exit(1);
 	}
 
-	interactive = !show && !clear &&!newLabel[0] && 
+	interactive = !show && !clear &&!newLabel[0] &&
 		(set_serial == SER_NONE);
 	if(!clear && !newLabel[0]) {
 		isRop = &isRo;
@@ -153,7 +181,7 @@ void mlabel(int argc, char **argv, int type)
 	if(isRo) {
 		show = 1;
 		interactive = 0;
-	}	    
+	}	
 	if(!RootDir) {
 		fprintf(stderr, "%s: Cannot initialize drive\n", argv[0]);
 		exit(1);
@@ -174,7 +202,7 @@ void mlabel(int argc, char **argv, int type)
 			printf(" Volume label is %s (abbr=%s)\n",
 			       longname, shortname);
 		else
-			printf(" Volume label is %s\n", shortname);
+			printf(" Volume label is %s\n",  shortname);
 
 	}
 
@@ -190,7 +218,7 @@ void mlabel(int argc, char **argv, int type)
 	if((!show || newLabel[0]) && !isNotFound(&entry)){
 		/* if we have a label, wipe it out before putting new one */
 		if(interactive && newLabel[0] == '\0')
-			if(ask_confirmation("Delete volume label (y/n): ",0,0)){
+			if(ask_confirmation("Delete volume label (y/n): ")){
 				FREE(&RootDir);
 				exit(0);
 			}
@@ -200,14 +228,14 @@ void mlabel(int argc, char **argv, int type)
 
 	if (newLabel[0] != '\0') {
 		ch.ignore_entry = 1;
-		result = mwrite_one(RootDir,newLabel,0,labelit,NULL,&ch) ? 
+		result = mwrite_one(RootDir,newLabel,0,labelit,NULL,&ch) ?
 		  0 : 1;
 	}
 
 	have_boot = 0;
 	if( (!show || newLabel[0]) || set_serial != SER_NONE) {
 		Fs = GetFs(RootDir);
-		have_boot = (force_read(Fs,(char *)&boot,0,sizeof(boot)) == 
+		have_boot = (force_read(Fs,(char *)&boot,0,sizeof(boot)) ==
 			     sizeof(boot));
 	}
 
@@ -218,15 +246,19 @@ void mlabel(int argc, char **argv, int type)
 	}
 
 	if(!show || newLabel[0]){
-
+		dos_name_t dosname;
+		const char *shrtLabel;
+		doscp_t *cp;
 		if(!newLabel[0])
-			strncpy(shortname, "NO NAME    ",11);
+			shrtLabel = "NO NAME    ";
 		else
-			label_name(newLabel, verbose, &mangled, shortname);
+			shrtLabel = newLabel;
+		cp = GET_DOSCONVERT(Fs);
+		label_name(cp, shrtLabel, verbose, &mangled, &dosname);
 
 		if(have_boot && boot.descr >= 0xf0 &&
 		   labelBlock->dos4 == 0x29) {
-			strncpy(labelBlock->label, shortname, 11);
+			strncpy(labelBlock->label, dosname.base, 11);
 			need_write_boot = 1;
 
 		}
