@@ -1,4 +1,19 @@
-/*
+/*  Copyright 1997,2000-2002,2009 Alain Knaff.
+ *  This file is part of mtools.
+ *
+ *  Mtools is free software: you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation, either version 3 of the License, or
+ *  (at your option) any later version.
+ *
+ *  Mtools is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with Mtools.  If not, see <http://www.gnu.org/licenses/>.
+ *
  * mcopy.c
  * Copy an MSDOS files to and from Unix
  *
@@ -44,13 +59,14 @@ static int unix_showfat(MainParam_t *mp)
 }
 
 
-static void usage(void)
+static void usage(int ret) NORETURN;
+static void usage(int ret)
 {
 	fprintf(stderr,
 		"Mtools version %s, dated %s\n", mversion, mdate);
 	fprintf(stderr,
 		"Usage: %s files\n", progname);
-	exit(1);
+	exit(ret);
 }
 
 void mshowfat(int argc, char **argv, int mtype)
@@ -63,16 +79,23 @@ void mshowfat(int argc, char **argv, int mtype)
 	init_clash_handling(& arg.ch);
 
 	/* get command line options */
-	while ((c = getopt(argc, argv, "")) != EOF) {
+	if(helpFlag(argc, argv))
+		usage(0);
+	while ((c = getopt(argc, argv, "i:h")) != EOF) {
 		switch (c) {
+			case 'i':
+				set_cmd_line_image(optarg, 0);
+				break;
+			case 'h':
+				usage(0);
 			case '?':
-				usage();
+				usage(1);
 				break;
 		}
 	}
 
 	if (argc - optind < 1)
-		usage();
+		usage(1);
 
 	/* only 1 file to copy... */
 	init_mp(&arg.mp);
